* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS LISTING CLOSE;

***********************************************************************************;
*******   MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR CHAPTER 10a MODELS    *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=, FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable;
* PredFewer = Name of OUTPM=work.= data file of predicted outcomes for nested model;
* PredMore =  Name of OUTPM=work.= data file of predicted outcomes for comparison model;
%MACRO TotalR2(DV=, PredFewer=, PredMore=);
PROC CORR NOPRINT NOSIMPLE DATA=&PredFewer. OUTP=CorrFewer; VAR pred &DV.; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredMore.  OUTP=CorrMore;  VAR pred &DV.; RUN;
DATA CorrFewer; LENGTH Name $30.; SET CorrFewer; Name="&PredFewer."; RUN;
DATA CorrMore;  LENGTH Name $30.; SET CorrMore;  Name="&PredMore.";  RUN;
DATA CorrCompare; LENGTH Name $30.; SET CorrFewer CorrMore; 
     PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; DROP Pred; RUN;
DATA CorrCompare; SET CorrCompare; TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
TITLE9 "Total R2 (% Reduction) for &PredFewer. vs. &PredMore.";
PROC PRINT NOOBS DATA=CorrCompare; RUN; TITLE9; 
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested model;
* CovMore =  Name of ODS CovParms table for comparison model;
%MACRO PseudoR2(NCov=, CovFewer=, CovMore=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
     THEN DELETE; RUN;
TITLE9 "PsuedoR2 (% Reduction) for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******  BEGIN DATA MANIPULATION FOR EXAMPLE 4A TIME-VARYING PREDICTORS     *******;
*******                 CHANGE "filesave" to your directory                 *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
%LET filesave= C:\Dropbox\23_PSQF7375_AdvLong\PSQF7375_AdvLong_Example4a;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";

* Import Example4a long data from excel;
PROC IMPORT DATAFILE="&filesave.\AdvLong_Example4a.xlsx" OUT=work.Example4a DBMS=XLSX REPLACE; 
     GETNAMES=YES;
RUN;

* Sort by ID, make person mean mood by averaging across rows -- save as new dataset;
PROC SORT DATA=work.Example4a; BY ID; RUN;
PROC MEANS NOPRINT DATA=work.Example4a; BY ID; VAR negmood;
     OUTPUT OUT=work.PersonMeans MEAN(negmood)=PMnegmood; RUN; 

* SAS code to merge in person means and center predictors;
DATA work.Example4a; MERGE work.Example4a work.PersonMeans; BY ID;
* Level-2 person mean (uncentered because mean=0, SD=1);
  PMnm0 = PMnegmood - 0;  LABEL PMnm0 = "PMnm0: Person Mean Negative Mood (0=0)";
* Level-1 predictor to use with PERSON-MEAN-CENTERING;
  WPnm  = negmood - PMnegmood; LABEL wpnm = "WPnm: Within-Person Negative Mood (0=PM)";
* Level-1 predictor to use with GRAND-MEAN-CENTERING;
  TVnm0 = negmood - 0; LABEL TVnm0 = "TVnm0: Time-Varying Negative Mood (0=0)";
* Binary gender already exists; 
  LABEL sexMW = "Participant Sex (0=M, 1=W)"; 
RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\AdvLong_Example4a_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE; 

******************************************************************************;
******                  PART 1: VARIANCE PARTITIONING                   ******;
******************************************************************************;

TITLE1 "SAS Model 1a: Empty Model for Daily Glucose Outcome";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;                           
     MODEL lglucAM = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR SUBJECT=ID TYPE=UN; * VCORR gives ICC;
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovEmpty InfoCrit=work.FitEmpty; * Save covparms, fit;
RUN; TITLE1;

TITLE1 "SAS Model 1b: Empty Model for Daily Negative Mood Predictor";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;
     MODEL negmood = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR SUBJECT=ID TYPE=UN; * VCORR gives ICC;
     REPEATED day / SUBJECT=ID TYPE=VC; 
RUN; TITLE1;


******************************************************************************;
******        PART 2: PERSON-MEAN-CENTERING (VARIABLE-CENTERING)        ******;
******************************************************************************;

TITLE1 "SAS Model 2a: Fixed Effects of Negative Mood using Person-MC";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;                           * Save predicted outcomes for total R2;
     MODEL lglucAM = WPnm PMnm0 / SOLUTION DDFM=Satterthwaite OUTPM=work.PredMood; 
     RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovMood InfoCrit=work.FitMood; * Save covparms, fit;
     CONTRAST "F-test of 2 Mood Slopes"           WPnm 1, PMnm0 1;
     ESTIMATE "L1 Within-Person Mood Effect"      WPnm  1;
     ESTIMATE "L2 Between-Person Mood Effect"     PMnm0 1;
     ESTIMATE "L2 Contextual Mood Effect"         PMnm0 1 WPnm -1;  
RUN; TITLE1;
* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=2, CovFewer=work.CovEmpty, CovMore=work.CovMood);
* Calculate Total R2 change relative to empty model;
%TotalR2(DV=lglucAM, PredFewer=work.PredMood, PredMore=work.PredMood);

TITLE1 "SAS Model 2b: Random Effect of WP Negative Mood using Person-MC";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;
     MODEL lglucAM = WPnm PMnm0 / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT WPnm / GCORR SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT InfoCrit=work.FitRandMood; * Save fit;
RUN; TITLE1;
* Calculate difference in model fit relative to fixed WPnm model;
%FitTest(FitFewer=work.FitMood, FitMore=work.FitRandMood);

TITLE1 "SAS Model 2c: Fixed Effects of Sex (0=M, 1=W) Moderating Person-MC Negative Mood";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;                           * Save predicted outcomes for total R2;
     MODEL lGlucAM = WPnm PMnm0 sexMW WPnm*sexMW PMnm0*sexMW 
                      / SOLUTION DDFM=Satterthwaite OUTPM=work.PredSex; 
     RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovSex; * Save covparms;
     CONTRAST "F-test of 3 Sex Slopes"   sexMW 1, WPnm*sexMW 1, PMnm0*sexMW 1; 
     ESTIMATE "Intercept: Men (at mood=0)"         intercept 1 sexMW 0;
     ESTIMATE "Intercept: Women (at mood=0)"       intercept 1 sexMW 1;
     ESTIMATE "Intercept: Women Diff (at mood=0)"  sexMW 1;
     ESTIMATE "L1 Within-Person Effect: Men"          WPnm 1 WPnm*sexMW 0;
     ESTIMATE "L1 Within-Person Effect: Women"        WPnm 1 WPnm*sexMW 1;
     ESTIMATE "L1 Within-Person Effect: Women Diff"   WPnm*sexMW 1;
     ESTIMATE "L2 Between-Person Effect: Men"         PMnm0 1 PMnm0*sexMW 0;
     ESTIMATE "L2 Between-Person Effect: Women"       PMnm0 1 PMnm0*sexMW 1;
     ESTIMATE "L2 Between-Person Effect: Women Diff"  PMnm0*sexMW 1;
     ESTIMATE "L2 Contextual Effect: Men"          WPnm -1 WPnm*sexMW  0 PMnm0 1 PMnm0*sexMW 0;
     ESTIMATE "L2 Contextual Effect: Women"        WPnm -1 WPnm*sexMW -1 PMnm0 1 PMnm0*sexMW 1;
     ESTIMATE "L2 Contextual Effect: Women Diff"   WPnm*sexMW -1 PMnm0*sexMW 1;
RUN; TITLE1;
* Calculate PseudoR2 relative to fixed-mood-only model;
%PseudoR2(NCov=2, CovFewer=work.CovMood, CovMore=work.CovSex);
* Calculate Total R2 change relative to fixed-mood-only model;
%TotalR2(DV=lGlucAM, PredFewer=work.PredMood, PredMore=work.PredSex);


******************************************************************************;
******        PART 3: GRAND-MEAN-CENTERING (CONSTANT-CENTERING)         ******;
******************************************************************************;

TITLE1 "SAS Model 3: Fixed Effect of TV Negative Mood only using Grand-MC";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;
     MODEL lglucAM = TVnm0 / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovSmush; * Save covparms;
RUN; TITLE1;
* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=2, CovFewer=work.CovEmpty, CovMore=work.CovSmush);

TITLE1 "SAS Model 3a: Fixed Effects of Negative Mood using Grand-MC";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;                           * Save predicted outcomes for total R2;
     MODEL lglucAM = TVnm0 PMnm0 / SOLUTION DDFM=Satterthwaite OUTPM=work.PredMood; 
     RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovMood InfoCrit=work.FitMood; * Save covparms, fit;
     CONTRAST "F-test of 2 Mood Slopes"           TVnm0 1, PMnm0 1;
     ESTIMATE "L1 Within-Person Mood Effect"      TVnm0 1;
     ESTIMATE "L2 Between-Person Mood Effect"     TVnm0 1 PMnm0 1;
     ESTIMATE "L2 Contextual Mood Effect"         PMnm0 1;  
RUN; TITLE1;
* Calculate PseudoR2 relative to smushed model;
%PseudoR2(NCov=2, CovFewer=work.CovSmush, CovMore=work.CovMood);
* Calculate Total R2 change relative to empty model;
%TotalR2(DV=lglucAM, PredFewer=work.PredMood, PredMore=work.PredMood);

TITLE1 "SAS Model 3b: Random Effect of TV Negative Mood using Grand-MC";
TITLE2 "FOR COMPARISON WITH MODEL 2B ONLY BECAUSE RANDOM SLOPE IS SMUSHED";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;
     MODEL lglucAM = TVnm0 PMnm0 / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT TVnm0 / GCORR SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT InfoCrit=work.FitTVRandMood; * Save fit;
RUN; TITLE1; TITLE2;
* Calculate difference in model fit relative to fixed TVnm0 model;
%FitTest(FitFewer=work.FitMood, FitMore=work.FitTVRandMood);

TITLE1 "SAS Model 3c: Fixed Effects of Sex (0=M, 1=W) by Grand-MC Negative Mood";
PROC MIXED DATA=work.Example4a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
     CLASS ID day;                           * Save predicted outcomes for total R2;
     MODEL lGlucAM = TVnm0 PMnm0 sexMW TVnm0*sexMW PMnm0*sexMW 
                      / SOLUTION DDFM=Satterthwaite OUTPM=work.PredSex; 
     RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
     REPEATED day / SUBJECT=ID TYPE=VC; 
     ODS OUTPUT CovParms=work.CovSex; * Save covparms;
     CONTRAST "F-test of 3 Sex Slopes"   sexMW 1, TVnm0*sexMW 1, PMnm0*sexMW 1; 
     ESTIMATE "Intercept: Men (at mood=0)"         intercept 1 sexMW 0;
     ESTIMATE "Intercept: Women (at mood=0)"       intercept 1 sexMW 1;
     ESTIMATE "Intercept: Women Diff (at mood=0)"  sexMW 1;
     ESTIMATE "L1 Within-Person Effect: Men"          TVnm0 1 TVnm0*sexMW 0;
     ESTIMATE "L1 Within-Person Effect: Women"        TVnm0 1 TVnm0*sexMW 1;
     ESTIMATE "L1 Within-Person Effect: Women Diff"   TVnm0*sexMW 1;
     ESTIMATE "L2 Between-Person Effect: Men"         TVnm0 1 TVnm0*sexMW 0 PMnm0 1 PMnm0*sexMW 0;
     ESTIMATE "L2 Between-Person Effect: Women"       TVnm0 1 TVnm0*sexMW 1 PMnm0 1 PMnm0*sexMW 1;
     ESTIMATE "L2 Between-Person Effect: Women Diff"  TVnm0*sexMW 1 PMnm0*sexMW 1;
     ESTIMATE "L2 Contextual Effect: Men"          PMnm0 1 PMnm0*sexMW 0;
     ESTIMATE "L2 Contextual Effect: Women"        PMnm0 1 PMnm0*sexMW 1;
     ESTIMATE "L2 Contextual Effect: Women Diff"   PMnm0*sexMW 1;
RUN; TITLE1;
* Calculate PseudoR2 relative to fixed-mood-only model;
%PseudoR2(NCov=2, CovFewer=work.CovMood, CovMore=work.CovSex);
* Calculate Total R2 change relative to fixed-mood-only model;
%TotalR2(DV=lGlucAM, PredFewer=work.PredMood, PredMore=work.PredSex);

* Close external file of saved results;
ODS RTF CLOSE;


*********************************************************************************;
* Trimming data to send just needed variables to Mplus;
DATA work.Example4aMplus; * RETAIN re-orders variables as listed;
     RETAIN ID lGlucAM TVnm0 WPnm PMnm0 sexMW;
     SET work.Example4a;
     * Telling it which variables to keep -- handy to use this in Mplus;
     KEEP ID lGlucAM TVnm0 WPnm PMnm0 sexMW;
     * Replace any missing values with -999;
     ARRAY avars(6) ID lGlucAM TVnm0 WPnm PMnm0 sexMW;
          DO i=1 TO 6; IF avars(i)=. THEN avars(i)=-999; END; DROP i; 
RUN;

* Export to .csv for use in Mplus MLM syntax;
PROC EXPORT DATA=work.Example4aMplus OUTFILE= "&filesave.\AdvLong_Example4a.csv" 
     DBMS=CSV REPLACE; PUTNAMES=NO; RUN;
